//=============================================================================
// TextDictionary.js v1.0.0
//
// Copyright (c) 2017 yatricia
// Released under the MIT License
// http://opensource.org/licenses/mit-license.php
//
// 更新履歴
// 1.0.0 2017/02/21 初版
//=============================================================================

/*:
 * @plugindesc ゲーム中で扱うテキストをまとめて管理するためのプラグイン
 * @author yatricia
 *
 * @help
 * 文章及び選択肢で\tという制御文字が使えるようになります。
 * \t[キー名]とすることで辞書内の対応するテキストに置き換えられます。
 * 辞書の定義はこのjsファイル中で直接行ってください。
 *
 * テキスト内に他の制御文字を使うことができます。
 * （\\N[1]のように\を二つ書く必要があります。）
 *
 * テキスト内に%1、%2、……のように書いておいて、
 * 呼び出し側を\t[キー,パラメタ1,パラメタ2]のようにすると
 * 該当部分を置き換えることができます。
 *
 * スクリプトや他のプラグインから使う場合はTextManager.tを直接使ってください。
 * ※このプラグインはコアスクリプトのTextManagerクラスにtメソッドを追加します。
 *
 * このプラグインには、プラグインコマンドはありません。
 * このプラグインは、MITライセンスのもとで公開されています。
 */
/* globals TextManager, Game_Message */
(function () {
    'use strict';

    // 辞書定義
    var dictionary = {
        'map': {
            '001': {
                '女性_1': 'こんにちは、\\N[1]。名前の埋め込みや',
                '女性_2': 'アイコン\\i[165]なども通常通り使えます。',
                '女性_3': 'ただし、円マークは二つ必要です。',
                '男_1': 'スクリプトからの使い方',
                '男_2': '\\i[%1]制御文字(\\\\)とパラメータ(%)の組み合わせ',
                'じいさん': '%1が%2で%3じゃ！'
            },
        },
        'monster': {
            'name': '俺は%1だ！',
        },
        'choice': {
            'yes': 'もちろんだ！',
            'no': 'だが断る'
        },
        'item': {
            'ride': '%1が落ちている',
            'select': '拾う？',
            '手に入れた': '%1を手に入れた。',
            '無視した': 'これはいらない。'
        }
    };

    // スクリプトからはこのメソッドを直接使えばよい
    TextManager.t = function(key) {
        var dic = dictionary;
        for (var k of key.split('.')) {
            dic = dic[k];
            if (dic === undefined) {
                break;
            }
        }

        // 2つ目以降の引数をformatに渡す
        var args = Array.prototype.slice.call(arguments, 1);
        //console.log(`${key} -> ${dic} (${args})`);
        return dic ? String.prototype.format.apply(dic, args) : (key + ' is not found.');
    };

    // メッセージ追加時に\tを展開するようにする
    var _Game_Message_add = Game_Message.prototype.add;
    Game_Message.prototype.add = function(text) {
        _Game_Message_add.call(this, replaceText(text));
    };

    // 選択肢追加時に\tを展開するようにする
    var _Game_Message_setChoices = Game_Message.prototype.setChoices;
    Game_Message.prototype.setChoices = function(choices, defaultType, cancelType) {
        _Game_Message_setChoices.call(this, choices.map(replaceText), defaultType, cancelType);
    };

    // 文字列中の\t部分を辞書のデータで置き換えた新しい文字列を返す
    var replaceText = function(text) {
        return text.replace(/\\t\[(.+?)\]/g, function(all, match) {
            return TextManager.t.apply(TextManager, match.split(','));
        });
    };
})();
